@@ -2,23 +2,23 @@
 
 ;;;;
 ;;;; Copyright (C) 2016 - 2017
-;;;; David Pirotte <david at altosw dot be>
+;;;; Free Software Foundation, Inc.
 
-;;;; This file is part of Guile-CV
+;;;; This file is part of GNU Guile-CV.
 
-;;;; Guile-CV is free software; you can redistribute it and/or modify it
-;;;; under the terms of the GNU Lesser General Public License as
+;;;; GNU Guile-CV is free software; you can redistribute it and/or
+;;;; modify it under the terms of the GNU General Public License as
 ;;;; published by the Free Software Foundation; either version 3 of the
 ;;;; License, or (at your option) any later version.
 
-;;;; Guile-CV is distributed in the hope that it will be useful, but
+;;;; GNU Guile-CV is distributed in the hope that it will be useful, but
 ;;;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;; Lesser General Public License for more details.
+;;;; General Public License for more details.
 
-;;;; You should have received a copy of the GNU Lesser General Public
-;;;; License along with Guile-CV.  If not, see
-;;;; <https://www.gnu.org/licenses/lgpl.html>.
+;;;; You should have received a copy of the GNU General Public License
+;;;; along with GNU Guile-CV.  If not, see
+;;;; <https://www.gnu.org/licenses/gpl.html>.
 ;;;;
 
 ;;; Commentary:
@@ -39,7 +39,7 @@
   #:use-module (cv idata)
   #:use-module (cv imgproc)
   #:use-module (cv segmentation)
-  #:use-module (cv properties)
+  #:use-module (cv features)
   
   #:duplicates (merge-generics
 		replace
@@ -48,23 +48,17 @@
 		last)
 
   #:export (#;im-map
-	    im-rgb->grey
+	    im-rgb->gray
             im-rgba->rgb
-            im-rgba->grey
+            im-rgba->gray
 	    im-threshold
-            im-add
-            im-add-channel
-            im-substract
-            im-substract-channel
-            im-multiply
-            im-multiply-channel
-            im-divide
-            im-divide-channel
 	    im-and
 	    im-or
 	    im-complement
 	    im-min
 	    im-max
+            im-inverse
+            im-inverse-channel
             im-transpose
             im-transpose-channel
             im-normalize
@@ -74,14 +68,21 @@
             im-particle-clean))
 
 
-#;(g-export )
+(g-export im-add
+          im-add-channel
+          im-subtract
+          im-subtract-channel
+          im-multiply
+          im-multiply-channel
+          im-divide
+          im-divide-channel)
 
 
 ;;;
 ;;; Guile-CV additional API
 ;;;
 
-(define (im-rgb->grey-1 c r g b i mini maxi total n-cell)
+(define (im-rgb->gray-1 c r g b i mini maxi total n-cell)
   (if (= i n-cell)
       (list mini maxi (round (/ total n-cell)) n-cell)
       (let ((k (/ (+ (f32vector-ref r i)
@@ -89,14 +90,14 @@
 		     (f32vector-ref b i))
 		  3)))
 	(f32vector-set! c i k)
-	(im-rgb->grey-1 c r g b
+	(im-rgb->gray-1 c r g b
 			(+ i 1)
 			(float-round (min mini k) 1)
 			(float-round (max maxi k) 1)
 			(+ total k)
 			n-cell))))
 
-(define (im-rgb->grey image)
+(define (im-rgb->gray image)
   (match image
     ((width height n-chan idata)
      (case n-chan
@@ -106,11 +107,11 @@
 	(match idata
 	  ((r g b)
 	   (let* ((c (im-make-channel width height))
-		  (vals (im-rgb->grey-1 c r g b 0 0 0 0 (* width height))))
+		  (vals (im-rgb->gray-1 c r g b 0 0 0 0 (* width height))))
 	     (values (list width height 1 (list c))
 		     vals)))))
        (else
-	(error "Not an RGB (nor a GREY) image."))))))
+	(error "Not an RGB (nor a GRAY) image."))))))
 
 #!
 Source => Target = (BGColor + Source) =
@@ -142,7 +143,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
           (else
            (error "Not an RGBA image."))))))
     (else
-     (error "Invalid background colour: " bg))))
+     (error "Invalid background color: " bg))))
 
 (define* (im-rgba-channel->rgb-channel c width height a-norm #:key (bg 0.0))
   (let ((c-norm (im-normalize-channel c width height))
@@ -160,16 +161,16 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                          255.0)))
     to))
 
-(define* (im-rgba->grey image #:key (bg '(0.0 0.0 0.0)))
+(define* (im-rgba->gray image #:key (bg '(0.0 0.0 0.0)))
   (match image
     ((_ _ n-chan idata)
      (case n-chan
        ((3 1)
-        (im-rgb->grey image))
+        (im-rgb->gray image))
        ((4)
-        (im-rgb->grey (im-rgba->rgb image #:bg bg)))
+        (im-rgb->gray (im-rgba->rgb image #:bg bg)))
        (else
-	(error "Not an RGBA (nor an RGB neither a GREY) image."))))))
+	(error "Not an RGBA (nor an RGB neither a GRAY) image."))))))
 
 (define* (im-threshold image threshold #:key (bg 'dark) (prec 1.0e-4))
   (if (and (>= threshold 0.0)
@@ -178,10 +179,10 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                ((_ _ n-chan _)
                 (case n-chan
                   ((1) image)
-                  ((3) (im-rgb->grey image))
-                  ((4) (im-rgba->grey image))
+                  ((3) (im-rgb->gray image))
+                  ((4) (im-rgba->gray image))
                   (else
-                   (error "Not a GREY, RGB, nor an RGBA image.")))))
+                   (error "Not a GRAY, RGB, nor an RGBA image.")))))
 	((width height n-chan idata)
 	 (match idata
 	   ((c)
@@ -199,85 +200,123 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
 		  (f32vector-set! c-copy i 255.0))))))))
       (error "Invalid threshold: " threshold)))
 
-(define (im-add image val)
+(define (im-scalar-op image val op)
   (match image
     ((width height n-chan idata)
      (list width height n-chan
            (let ((map-proc (if (and (> n-chan 1)
                                     (%use-par-map)) par-map map)))
 	     (map-proc (lambda (channel)
-			 (im-add-channel channel width height val))
-	       idata))))))
+			 (im-scalar-channel-op channel width height val op))
+                       idata))))))
 
-(define (im-add-channel channel width height val)
+(define (im-scalar-channel-op channel width height val op)
   (let ((to (im-make-channel width height))
         (n-cell (* width height)))
     (do ((i 0
 	    (+ i 1)))
-	((= i n-cell))
-      (let ((new-val (+ (f32vector-ref channel i) val)))
-        (f32vector-set! to i (if (float>=? new-val 255.0) 255.0 new-val))))
-    to))
+	((= i n-cell) to)
+      (f32vector-set! to i (op (f32vector-ref channel i) val)))))
 
-(define (im-substract image val)
+(define (im-matrix-op image img-2 op)
   (match image
     ((width height n-chan idata)
-     (list width height n-chan
-           (let ((map-proc (if (and (> n-chan 1)
-                                    (%use-par-map)) par-map map)))
-	     (map-proc (lambda (channel)
-			 (im-substract-channel channel width height val))
-	       idata))))))
+     (match img-2
+       ((width-2 height-2 n-chan-2 idata-2)
+        (if (and (= width width-2)
+                 (= height height-2)
+                 (= n-chan n-chan-2))
+            (list width height n-chan
+                  (let ((map-proc (if (and (> n-chan 1)
+                                           (%use-par-map)) par-map map)))
+                    (map-proc (lambda (channels)
+                                (match channels
+                                  ((c1 c2)
+                                   (im-matrix-channel-op c1 width height c2 op))))
+                              (zip idata idata-2))))
+            (error "Size missmatch.")))))))
 
-(define (im-substract-channel channel width height val)
+(define (im-matrix-channel-op channel width height channel-2 op)
   (let ((to (im-make-channel width height))
         (n-cell (* width height)))
     (do ((i 0
 	    (+ i 1)))
-	((= i n-cell))
-      (let ((new-val (- (f32vector-ref channel i) val)))
-        (f32vector-set! to i (if (float<=? new-val 0.0) 0.0 new-val))))
-    to))
+	((= i n-cell) to)
+    (f32vector-set! to i (op (f32vector-ref channel i)
+                             (f32vector-ref channel-2 i))))))
 
-(define (im-multiply image val)
-  (match image
-    ((width height n-chan idata)
-     (list width height n-chan
-           (let ((map-proc (if (and (> n-chan 1)
-                                    (%use-par-map)) par-map map)))
-	     (map-proc (lambda (channel)
-			 (im-multiply-channel channel width height val))
-	       idata))))))
+(define (im-matrix-multdiv-op img-1 img-2 op)
+  ;; The product is defined only if the number of columns in img-1 is
+  ;; equal to the number of rows in img-2.
+  ;; The division is a multiplication by the inverse (of img-2)
+  (match img-1
+    ((width-1 height-1 n-chan-1 idata-1)
+     (match img-2
+       ((width-2 height-2 n-chan-2 idata-2)
+        (if (= width-1 height-2)
+            (list width-2 height-1 n-chan-1
+                  (let ((map-proc (if (and (> n-chan-1 1)
+                                           (%use-par-map)) par-map map)))
+                    (map-proc (lambda (channels)
+                                (match channels
+                                  ((c1 c2)
+                                   (op c1 width-1 height-1 c2 width-2))))
+                              (zip idata-1 idata-2))))
+            (error "Size missmatch.")))))))
 
-(define (im-multiply-channel channel width height val)
-  (let ((to (im-make-channel width height))
-        (n-cell (* width height)))
-    (do ((i 0
-	    (+ i 1)))
-	((= i n-cell))
-      (let ((new-val (* (f32vector-ref channel i) val)))
-        (f32vector-set! to i (if (float>=? new-val 255.0) 255.0 new-val))))
-    to))
 
-(define (im-divide image val)
-  (match image
-    ((width height n-chan idata)
-     (list width height n-chan
-           (let ((map-proc (if (and (> n-chan 1)
-                                    (%use-par-map)) par-map map)))
-	     (map-proc (lambda (channel)
-			 (im-divide-channel channel width height val))
-	       idata))))))
 
-(define (im-divide-channel channel width height val)
-  (let ((to (im-make-channel width height))
-        (n-cell (* width height)))
-    (do ((i 0
-	    (+ i 1)))
-	((= i n-cell))
-      (let ((new-val (/ (f32vector-ref channel i) val)))
-        (f32vector-set! to i (if (float<=? new-val 0.0) 0.0 new-val))))
-    to))
+(define-method (im-add image (val <number>))
+  (im-scalar-op image val +))
+
+(define-method (im-add-channel channel width height (val <number>))
+  (im-scalar-channel-op channel width height val +))
+
+(define-method (im-add image (img-2 <list>))
+  (im-matrix-op image img-2 +))
+
+(define-method (im-add-channel channel width height (channel-2 <uvec>))
+  (im-matrix-channel-op channel width height channel-2 +))
+
+
+(define-method (im-subtract image (val <number>))
+  (im-scalar-op image val -))
+
+(define-method (im-subtract-channel channel width height (val <number>))
+  (im-scalar-channel-op channel width height val -))
+
+(define-method (im-subtract image (img-2 <list>))
+  (im-matrix-op image img-2 -))
+
+(define-method (im-subtract-channel channel width height (channel-2 <uvec>))
+  (im-matrix-channel-op channel width height channel-2 -))
+
+
+(define-method (im-multiply image (val <number>))
+  (im-scalar-op image val *))
+
+(define-method (im-multiply-channel channel width height (val <number>))
+  (im-scalar-channel-op channel width height val *))
+
+(define-method (im-multiply img-1 (img-2 <list>))
+  (im-matrix-multdiv-op img-1 img-2 im-multiply-channel))
+
+(define-method (im-multiply-channel c1 width-1 height-1 (c2 <uvec>) width-2)
+  (f32vector-matrix-multiply c1 width-1 height-1 c2 width-2))
+
+
+(define-method (im-divide image (val <number>))
+  (im-scalar-op image val /))
+
+(define-method (im-divide-channel channel width height (val <number>))
+  (im-scalar-channel-op channel width height val /))
+
+(define-method (im-divide img-1 (img-2 <list>))
+  (im-matrix-multdiv-op img-1 img-2 im-divide-channel))
+
+(define-method (im-divide-channel c1 width-1 height-1 (c2 <uvec>) width-2)
+  (f32vector-matrix-multiply c1 width-1 height-1 (f32vector-inverse c2) width-2))
+
 
 (define (im-and . images)
   (match images
@@ -288,7 +327,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
 		 (apply = (cons height (im-collect rest 'height))))
 	    (let ((img-2 (im-copy image))
 		  (n-cell (* width height))
-		  (c-channels (im-collect (map im-rgb->grey rest) 'grey)))
+		  (c-channels (im-collect (map im-rgb->gray rest) 'gray)))
 	      (list width height n-chan
                     (let ((map-proc (if (and (> n-chan 1)
                                              (%use-par-map)) par-map map)))
@@ -313,7 +352,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
 		 (apply = (cons height (im-collect rest 'height))))
 	    (let ((img-2 (im-copy image))
 		  (n-cell (* width height))
-		  (c-channels (im-collect (map im-rgb->grey rest) 'grey)))
+		  (c-channels (im-collect (map im-rgb->gray rest) 'gray)))
 	      (list width height n-chan
                     (let ((map-proc (if (and (> n-chan 1)
                                              (%use-par-map)) par-map map)))
@@ -384,6 +423,19 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                               (im-fast-channel-ref channel i j width))))
     to))
 
+(define (im-inverse image)
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+	     (map-proc (lambda (channel)
+			 (im-inverse-channel channel width height))
+                       idata))))))
+
+(define (im-inverse-channel channel width height)
+  (f32vector-inverse channel))
+
 (define (im-normalize image)
   (match image
     ((width height n-chan idata)
@@ -417,17 +469,17 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
             ((_ _ _ l-idata)
              (match l-idata
                ((l-channel)
-                (let* ((properties (im-properties image l-image #:n-label n-label))
-                       (n-property (length properties))
-                       (to-scrap (fold (lambda (property i prev)
-                                         (match property
+                (let* ((features (im-features image l-image #:n-label n-label))
+                       (n-feature (length features))
+                       (to-scrap (fold (lambda (feature i prev)
+                                         (match feature
                                            ((area . rest)
                                             (if (pred area val)
                                                 (cons i prev)
                                                 prev))))
                                        '()
-                                       properties
-                                       (iota n-property))))
+                                       features
+                                       (iota n-feature))))
                   (list width height 1
                         (list (im-scrap-channel channel l-channel width height
                                                 to-scrap n-label))))))))))
@@ -477,7 +529,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                             (f32vector-ref channel i)))))
     to))
 
-(define* (im-particles image properties #:key (clean #t))
+(define* (im-particles image features #:key (clean #t))
   (let ((map-proc (if (%use-par-map) par-map map)))
     (map-proc (lambda (prop)
                 (match prop
@@ -487,7 +539,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                        (if clean
                            (im-particle-clean particle)
                            particle))))))
-              (cdr properties))))
+              (cdr features))))
 
 (define (im-particle-clean particle)
   (let* ((binary? (im-binary? particle))
@@ -510,7 +562,8 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                                                         (cons i prev)
                                                         prev))))
                                                '()
-                                               (im-properties binary label-im #:n-label n-label)
+                                               (im-features binary label-im
+                                                            #:n-label n-label)
                                                (iota (+ n-label 1))))
                               (cache (scrap-cache to-remove n-label)))
                          (do ((i 0
