@@ -2,23 +2,23 @@
 
 ;;;;
 ;;;; Copyright (C) 2016 - 2017
-;;;; David Pirotte <david at altosw dot be>
+;;;; Free Software Foundation, Inc.
 
-;;;; This file is part of Guile-CV
+;;;; This file is part of GNU Guile-CV.
 
-;;;; Guile-CV is free software; you can redistribute it and/or modify it
-;;;; under the terms of the GNU Lesser General Public License as
+;;;; GNU Guile-CV is free software; you can redistribute it and/or
+;;;; modify it under the terms of the GNU General Public License as
 ;;;; published by the Free Software Foundation; either version 3 of the
 ;;;; License, or (at your option) any later version.
 
-;;;; Guile-CV is distributed in the hope that it will be useful, but
+;;;; GNU Guile-CV is distributed in the hope that it will be useful, but
 ;;;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;;; Lesser General Public License for more details.
+;;;; General Public License for more details.
 
-;;;; You should have received a copy of the GNU Lesser General Public
-;;;; License along with Guile-CV.  If not, see
-;;;; <https://www.gnu.org/licenses/lgpl.html>.
+;;;; You should have received a copy of the GNU General Public License
+;;;; along with GNU Guile-CV.  If not, see
+;;;; <https://www.gnu.org/licenses/gpl.html>.
 ;;;;
 
 ;;; Commentary:
@@ -86,7 +86,7 @@
 	  im-n-channel
 	  im-channels
 
-	  im-grey?
+	  im-gray?
 	  im-rgb?)
 
 
@@ -171,7 +171,7 @@
 		    (float-member 255.0 vals)))))
 	  #t))))
 
-(define-method (im-grey? (image <list>))
+(define-method (im-gray? (image <list>))
   (match image
     ((width height n-chan idata)
      (= n-chan 1))))
@@ -188,14 +188,14 @@
 	((width height n-chan _)
 	 (if (and (apply = (cons width (im-collect rest 'width)))
 		  (apply = (cons height (im-collect rest 'height)))
-		  (apply = (im-collect rest 'n-channel)))
+		  (apply = (cons n-chan (im-collect rest 'n-channel))))
 	     (catch 'exit
 	       (lambda ()
 		 (let ((n-cell (* width height)))
 		   (for-each (lambda (k)
 			       (let* ((chan-k (n-chan->symbol k))
 				      (channels (im-collect images chan-k)))
-				 (unless (f32vector-list=? channels prec)
+				 (unless (f32vector-list=? channels #:prec prec)
 				   (throw 'exit #f))))
 		       (iota n-chan))
 		   #t))
@@ -343,7 +343,7 @@
     ((n-channel) (map im-n-channel images))
     ((size) (map im-size images))
     ((channels) (map im-channels images))
-    ((chan-0 grey red) (collect-channel images 0))
+    ((chan-0 gray red) (collect-channel images 0))
     ((chan-1 green) (collect-channel images 1))
     ((chan-2 blue) (collect-channel images 2))
     (else
